#ifndef ARCHITECTURE_H_
#define ARCHITECTURE_H_

#include <msp/datafile/loader.h>
#include "buildinfo.h"
#include "pattern.h"

class Builder;

/**
Stores information about an architecture.  This includes CPU type, model and
bitness and operating system.
*/
class Architecture
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Architecture>
	{
	public:
		Loader(Architecture &);
	};

	typedef std::list<Pattern> PatternList;

private:
	Builder &builder;
	std::string type;
	std::string cpu;
	std::string system;
	unsigned bits;
	std::string name;
	bool native;
	std::string cross_prefix;
	PatternList sharedlib_patterns;
	PatternList staticlib_patterns;
	PatternList executable_patterns;

public:
	Architecture(Builder &b, const std::string &spec);

	const std::string &get_type() const { return type; }
	const std::string &get_name() const { return name; }
	const std::string &get_system() const { return system; }
	unsigned get_bits() const { return bits; }
	const std::string &get_cpu() const { return cpu; }
	bool match_name(const std::string &) const;
	bool is_native() const { return native; }
	bool is_cross() const { return !cross_prefix.empty(); }

	const std::string &get_cross_prefix() const { return cross_prefix; }

	const PatternList &get_shared_library_patterns() const { return sharedlib_patterns; }
	const PatternList &get_static_library_patterns() const { return staticlib_patterns; }
	const PatternList &get_executable_patterns() const { return executable_patterns; }

private:
	std::string resolve_alias(const std::string &) const;
	void parse_specification(const std::string &);
};

#endif
